"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hook = exports.net = void 0;
const krevent_1 = require("krevent");
const netevent = require("./netevent");
const native_1 = require("./native");
const PacketId = require("./packetId");
const common_1 = require("./common");
class CommandEventImpl {
    constructor(command, networkIdentifier) {
        this.command = command;
        this.networkIdentifier = networkIdentifier;
        this.isModified = false;
    }
    setCommand(command) {
        this.isModified = true;
        this.command = command;
    }
}
class UserCommandEvents extends krevent_1.EventEx {
    constructor() {
        super(...arguments);
        this.listener = (ptr, networkIdentifier, packetId) => {
            ptr.move(0x28);
            const command = ptr.readCxxString();
            const ev = new CommandEventImpl(command, networkIdentifier);
            if (this.fire(ev) === common_1.CANCEL)
                return common_1.CANCEL;
            if (ev.isModified) {
                ptr.move(-0x20);
                ptr.writeCxxString(ev.command);
            }
        };
    }
    onStarted() {
        netevent.before(PacketId.CommandRequest).on(this.listener);
    }
    onCleared() {
        netevent.before(PacketId.CommandRequest).remove(this.listener);
    }
}
class HookCommandEvents extends krevent_1.EventEx {
    constructor() {
        super(...arguments);
        this.listener = (command, originName) => {
            return this.fire(command, originName);
        };
    }
    onStarted() {
        native_1.setOnCommandListener(this.listener);
    }
    onCleared() {
        native_1.setOnCommandListener(null);
    }
}
exports.net = new UserCommandEvents();
exports.hook = new HookCommandEvents();
//# sourceMappingURL=command.js.map