"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MariaDB = exports.MariaDBTransaction = void 0;
const native = require("./native");
class SqlError extends Error {
    constructor(message, errno) {
        super(message);
        this.errno = errno;
    }
}
if (!Promise.prototype.finally) {
    Promise.prototype.finally = function (onfinally) {
        function voiding(value) {
            return __awaiter(this, void 0, void 0, function* () {
                if (!onfinally)
                    return;
                onfinally();
                return value;
            });
        }
        return this.then(voiding, voiding);
    };
}
/**
 * @deprecated use node.js instead
 */
class MariaDBTransaction {
    constructor(db) {
        this.db = db;
    }
    query(query) {
        return new Promise((resolve, reject) => {
            this.db.query(query, (error, fieldCount) => {
                if (error)
                    reject(Error(error));
                else
                    resolve(fieldCount);
            });
        });
    }
    fetch() {
        return new Promise(resolve => {
            this.db.fetch((row) => {
                resolve(row);
            });
        });
    }
}
exports.MariaDBTransaction = MariaDBTransaction;
/**
 * @deprecated use node.js instead
 */
class MariaDB {
    constructor(host, username, password, db, port) {
        this.acEnabled = true;
        var resolve;
        var reject;
        this.worker = new Promise((_resolve, _reject) => {
            resolve = _resolve;
            reject = _reject;
        });
        this.db = new native.MariaDB((error, errno) => {
            if (error) {
                reject(new SqlError(error, errno));
            }
            else {
                resolve();
            }
        }, host, username, password, db, port);
        this.tran = new MariaDBTransaction(this.db);
    }
    get [Symbol.toStringTag]() {
        return 'MariaDB';
    }
    _autocommit(enabled) {
        if (this.acEnabled === enabled)
            return;
        this.acEnabled = enabled;
        this.db.autocommit(enabled);
    }
    transaction(func) {
        const ret = this.worker.then(() => {
            this._autocommit(false);
        }).then(() => func(this.tran));
        this.worker = ret.then(() => {
            this.db.commit();
        }, () => {
            this.db.rollback();
        });
        return ret;
    }
    query(query, suppressError) {
        this.worker.then(() => {
            this._autocommit(true);
            this.db.query(query, !suppressError);
        });
    }
    execute(query) {
        return this.transaction(() => __awaiter(this, void 0, void 0, function* () {
            this.db.query(query);
            const out = [];
            let res;
            while (res = yield this.tran.fetch()) {
                out.push(res);
            }
            this.db.closeResult();
            return out;
        }));
    }
}
exports.MariaDB = MariaDB;
//# sourceMappingURL=db.js.map