import { fs } from "./native";
import { Bufferable, Encoding, TypeFromEncoding } from "./common";
import Event from "krevent";
import File = fs.File;
export interface ReadReport<T> {
    data: T;
    readedBytes: number;
    failed: Uint8Array | null;
}
export interface WriteReport {
    writedBytes: number;
    failed: Uint8Array | null;
}
declare module './native' {
    namespace fs {
        /**
         * @deprecated Use node.js
         */
        interface File {
            /**
             * Read as a string
             * @param offset position from begin of file
             * @param size reading size
             */
            readString<T extends Encoding = Encoding.Utf8>(offset: number, size: number, encoding?: T): Promise<ReadReport<TypeFromEncoding<T>>>;
            /**
             * Write file
             * @param offset position from begin of file
             * @param buffer buffer for writing
             */
            writeString(offset: number, buffer: string, encoding?: Encoding): Promise<WriteReport>;
            /**
            * Read as a buffer
            * @param offset position from begin of file
            * @param size reading size
            */
            readBuffer(offset: number, size: number): Promise<Uint8Array>;
            /**
             * Write file
             * @param offset position from begin of file
             * @param buffer buffer for writing
             */
            writeBuffer(offset: number, buffer: Bufferable): Promise<number>;
        }
    }
}
export { File, File as NativeFile };
export declare const readFileSync: typeof fs.readFileSync;
export declare const writeFileSync: typeof fs.writeFileSync;
export declare const appendFileSync: typeof fs.appendFileSync;
export declare const deleteFileSync: typeof fs.deleteFileSync;
export declare const deleteRecursiveSync: typeof fs.deleteRecursiveSync;
export declare const copyFileSync: typeof fs.copyFileSync;
export declare const copyRecursiveSync: typeof fs.copyRecursiveSync;
export declare const mkdirSync: typeof fs.mkdirSync;
export declare const mkdirRecursiveSync: typeof fs.mkdirRecursiveSync;
export declare const cwd: typeof fs.cwd;
export declare const chdir: typeof fs.chdir;
/**
 * @deprecated use node.js
 */
export declare function writeFile(path: string, content: string, encoding?: Encoding): Promise<void>;
/**
 * @deprecated use node.js
 */
export declare function appendFile(path: string, content: string, encoding?: Encoding): Promise<void>;
/**
 * @deprecated use node.js
 */
export declare function readFile(path: string, encoding?: Encoding): Promise<string>;
/**
 * @deprecated use node.js
 */
export declare function resolve(path: string): string;
/**
 * @deprecated use node.js
 */
export declare function basename(path: string): string;
/**
 * @deprecated use node.js
 */
export declare function dirname(path: string): string;
/**
 * @deprecated use node.js
 */
export declare class DirectoryWatcher extends fs.Watcher {
    private static readonly all;
    readonly onCreated: Event<(path: string) => void>;
    readonly onDeleted: Event<(path: string) => void>;
    readonly onModified: Event<(path: string) => void>;
    readonly onRenamed: Event<(to: string, from: string) => void>;
    private reference;
    constructor(path: string);
    release(): void;
    static getInstance(path: string): DirectoryWatcher;
}
declare class WatchEvent {
    private readonly shared;
    private _onCreated;
    private _onDeleted;
    private _onModified;
    private _onRenamed;
    private closed;
    constructor(shared: DirectoryWatcher);
    setOnCreated(listener: ((name: string) => void) | null): void;
    setOnDeleted(listener: ((name: string) => void) | null): void;
    setOnModified(listener: ((name: string) => void) | null): void;
    setOnRenamed(listener: ((to: string, from: string) => void) | null): void;
    close(): void;
}
/**
 * @deprecated use node.js
 */
export declare function watch(path: string): WatchEvent;
/**
 * @deprecated use node.js
 */
export declare function watchDir(path: string): WatchEvent;
