"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchDir = exports.watch = exports.DirectoryWatcher = exports.dirname = exports.basename = exports.resolve = exports.readFile = exports.appendFile = exports.writeFile = exports.chdir = exports.cwd = exports.mkdirRecursiveSync = exports.mkdirSync = exports.copyRecursiveSync = exports.copyFileSync = exports.deleteRecursiveSync = exports.deleteFileSync = exports.appendFileSync = exports.writeFileSync = exports.readFileSync = exports.NativeFile = exports.File = void 0;
const native_1 = require("./native");
const krevent_1 = require("krevent");
var File = native_1.fs.File;
exports.File = File;
exports.NativeFile = File;
File.prototype.readBuffer = function (offset, size) {
    return new Promise((resolve, reject) => {
        this.read(offset, size, (err, data) => {
            if (err)
                reject(Error(err));
            else
                resolve(data);
        });
    });
};
File.prototype.readString = function (offset, size, encoding) {
    return new Promise((resolve, reject) => {
        this.read(offset, size, (err, data) => {
            const [decoded, readedlen] = native_1.decode(data, encoding);
            if (err)
                reject(Error(err));
            else
                resolve({
                    data: decoded,
                    readedBytes: data.length,
                    failed: data.length === readedlen ? null : data.subarray(readedlen),
                });
        });
    });
};
File.prototype.writeString = function (offset, buffer, encoding) {
    return new Promise((resolve, reject) => {
        const buf = native_1.encode(buffer, encoding);
        this.write(offset, buf, (err, bytes) => {
            if (err)
                reject(Error(err));
            else
                resolve({
                    writedBytes: bytes,
                    failed: buf.subarray(bytes)
                });
        });
    });
};
File.prototype.writeBuffer = function (offset, buffer) {
    return new Promise((resolve, reject) => {
        this.write(offset, buffer, (err, bytes) => {
            if (err)
                reject(Error(err));
            else
                resolve(bytes);
        });
    });
};
exports.readFileSync = native_1.fs.readFileSync;
exports.writeFileSync = native_1.fs.writeFileSync;
exports.appendFileSync = native_1.fs.appendFileSync;
exports.deleteFileSync = native_1.fs.deleteFileSync;
exports.deleteRecursiveSync = native_1.fs.deleteRecursiveSync;
exports.copyFileSync = native_1.fs.copyFileSync;
exports.copyRecursiveSync = native_1.fs.copyRecursiveSync;
exports.mkdirSync = native_1.fs.mkdirSync;
exports.mkdirRecursiveSync = native_1.fs.mkdirRecursiveSync;
exports.cwd = native_1.fs.cwd;
exports.chdir = native_1.fs.chdir;
/**
 * @deprecated use node.js
 */
function writeFile(path, content, encoding) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = new File(path, File.WRITE, File.CREATE_ALWAYS);
        try {
            let buf = native_1.encode(content, encoding);
            let offset = 0;
            while (buf.length !== 0) {
                const writed = yield file.writeBuffer(offset, buf);
                offset += writed;
                buf = buf.subarray(writed);
            }
            file.close();
        }
        catch (err) {
            file.close();
            throw err;
        }
    });
}
exports.writeFile = writeFile;
/**
 * @deprecated use node.js
 */
function appendFile(path, content, encoding) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = new File(path, File.WRITE, File.OPEN_ALWAYS);
        try {
            let buf = native_1.encode(content, encoding);
            while (buf.length !== 0) {
                const writed = yield file.writeBuffer(-1, buf);
                buf = buf.subarray(writed);
            }
            file.close();
        }
        catch (err) {
            file.close();
            throw err;
        }
    });
}
exports.appendFile = appendFile;
/**
 * @deprecated use node.js
 */
function readFile(path, encoding) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = new File(path, File.READ, File.OPEN_EXISTING);
        let out = '';
        let byteOffset = 0;
        let remained = null;
        try {
            for (;;) {
                let data = yield file.readBuffer(byteOffset, 8192);
                if (data.length == 0) {
                    file.close();
                    return out;
                }
                byteOffset += data.length;
                if (remained) {
                    const concat = new Uint8Array(remained.length + data.length);
                    concat.set(remained);
                    concat.set(data, remained.length);
                    data = concat;
                }
                const [decoded, decodedBytes] = native_1.decode(data, encoding);
                out += decoded;
                remained = data.subarray(decodedBytes);
            }
        }
        catch (err) {
            file.close();
            throw err;
        }
    });
}
exports.readFile = readFile;
/**
 * @deprecated use node.js
 */
function resolve(path) {
    path = path.replace(/\//g, '\\');
    let out;
    const dirs = path.split('\\');
    if (dirs[0] !== '' && !dirs[0].endsWith(':')) {
        out = exports.cwd().split(/\\/g);
    }
    else {
        out = [];
    }
    for (const dir of path.split('\\')) {
        if (dir.charAt(0) === '.') {
            if (dir.length === 1) {
                continue;
            }
            else if (dir.length === 2 && dir.charAt(1) === '.') {
                out.pop();
                continue;
            }
        }
        out.push(dir);
    }
    return out.join('\\');
}
exports.resolve = resolve;
/**
 * @deprecated use node.js
 */
function basename(path) {
    const idx = Math.max(path.lastIndexOf('\\'), path.lastIndexOf('/'));
    if (idx === -1)
        return path;
    return path.substr(idx + 1);
}
exports.basename = basename;
/**
 * @deprecated use node.js
 */
function dirname(path) {
    const idx = Math.max(path.lastIndexOf('\\'), path.lastIndexOf('/'));
    if (idx === -1)
        return '.';
    return path.substr(0, idx);
}
exports.dirname = dirname;
/**
 * @deprecated use node.js
 */
class DirectoryWatcher extends native_1.fs.Watcher {
    constructor(path) {
        super(path, false);
        this.onCreated = new krevent_1.default();
        this.onDeleted = new krevent_1.default();
        this.onModified = new krevent_1.default();
        this.onRenamed = new krevent_1.default();
        this.reference = 0;
        this.setOnCreated(name => this.onCreated.fire(name));
        this.setOnDeleted(name => this.onDeleted.fire(name));
        this.setOnModified(name => this.onModified.fire(name));
        this.setOnRenamed((to, from) => this.onRenamed.fire(to, from));
    }
    release() {
        console.assert(this.reference !== 0, "out of reference count");
        this.reference--;
        if (this.reference === 0) {
            this.close();
        }
    }
    static getInstance(path) {
        path = resolve(path);
        let watcher = DirectoryWatcher.all.get(path);
        if (!watcher) {
            watcher = new DirectoryWatcher(path);
            DirectoryWatcher.all.set(path, watcher);
        }
        watcher.reference++;
        return watcher;
    }
}
exports.DirectoryWatcher = DirectoryWatcher;
DirectoryWatcher.all = new Map();
class WatchEvent {
    constructor(shared) {
        this.shared = shared;
        this._onCreated = null;
        this._onDeleted = null;
        this._onModified = null;
        this._onRenamed = null;
        this.closed = false;
    }
    setOnCreated(listener) {
        if (this.closed)
            return;
        if (this._onCreated === listener)
            return;
        if (this._onCreated)
            this.shared.onCreated.remove(this._onCreated);
        if (listener)
            this.shared.onCreated.on(listener);
        this._onCreated = listener;
    }
    setOnDeleted(listener) {
        if (this.closed)
            return;
        if (this._onDeleted === listener)
            return;
        if (this._onDeleted)
            this.shared.onDeleted.remove(this._onDeleted);
        if (listener)
            this.shared.onDeleted.on(listener);
        this._onDeleted = listener;
    }
    setOnModified(listener) {
        if (this.closed)
            return;
        if (this._onModified === listener)
            return;
        if (this._onModified)
            this.shared.onModified.remove(this._onModified);
        if (listener)
            this.shared.onModified.on(listener);
        this._onModified = listener;
    }
    setOnRenamed(listener) {
        if (this.closed)
            return;
        if (this._onRenamed === listener)
            return;
        if (this._onRenamed)
            this.shared.onRenamed.remove(this._onRenamed);
        if (listener)
            this.shared.onRenamed.on(listener);
        this._onRenamed = listener;
    }
    close() {
        if (this.closed)
            return;
        this.closed = true;
        if (this._onCreated) {
            this.shared.onCreated.remove(this._onCreated);
            this._onCreated = null;
        }
        if (this._onDeleted) {
            this.shared.onDeleted.remove(this._onDeleted);
            this._onDeleted = null;
        }
        if (this._onModified) {
            this.shared.onModified.remove(this._onModified);
            this._onModified = null;
        }
        if (this._onRenamed) {
            this.shared.onRenamed.remove(this._onRenamed);
            this._onRenamed = null;
        }
        this.shared.release();
    }
}
class FilteredWatchEvent extends WatchEvent {
    constructor(shared, name) {
        super(shared);
        this.name = name;
    }
    setOnCreated(listener) {
        if (listener) {
            super.setOnCreated(name => {
                if (name !== this.name)
                    return;
                listener(name);
            });
        }
        else {
            super.setOnCreated(null);
        }
    }
    setOnDeleted(listener) {
        if (listener) {
            super.setOnDeleted(name => {
                if (name !== this.name)
                    return;
                listener(name);
            });
        }
        else {
            super.setOnDeleted(null);
        }
    }
    setOnModified(listener) {
        if (listener) {
            super.setOnModified(name => {
                if (name !== this.name)
                    return;
                listener(name);
            });
        }
        else {
            super.setOnModified(null);
        }
    }
    setOnRenamed(listener) {
        if (listener) {
            super.setOnRenamed((to, from) => {
                if (from !== this.name && to !== this.name)
                    return;
                listener(to, from);
            });
        }
        else {
            super.setOnRenamed(null);
        }
    }
}
/**
 * @deprecated use node.js
 */
function watch(path) {
    path = resolve(path);
    const watcher = DirectoryWatcher.getInstance(dirname(path));
    return new FilteredWatchEvent(watcher, basename(path));
}
exports.watch = watch;
/**
 * @deprecated use node.js
 */
function watchDir(path) {
    const watcher = DirectoryWatcher.getInstance(path);
    return new WatchEvent(watcher);
}
exports.watchDir = watchDir;
//# sourceMappingURL=fs.js.map