import { Type, Primitive, NativePointer, NetworkIdentifier, StaticPointer } from './native';
declare const Uint8: Primitive;
declare const Uint16: Primitive;
declare const Uint32: Primitive;
declare const Int8: Primitive;
declare const Int16: Primitive;
declare const Int32: Primitive;
declare const Float32: Primitive;
declare const Float64: Primitive;
declare module './native' {
    namespace NativePointer {
        let size: number;
        function get(ptr: NativePointer): NativePointer;
        function set(ptr: NativePointer, value: NativePointer): void;
    }
    namespace SharedPointer {
        let size: number;
        function get(ptr: NativePointer): SharedPointer;
        function set(ptr: NativePointer, value: SharedPointer): void;
    }
    namespace NetworkIdentifier {
        let size: number;
        function get(ptr: NativePointer): NetworkIdentifier;
        function set(ptr: NativePointer, value: NetworkIdentifier): void;
    }
    namespace Actor {
        function get(ptr: NativePointer): Actor;
        function set(ptr: NativePointer, value: Actor): void;
    }
}
export declare const CxxString: {
    get(ptr: NativePointer): string;
    set(ptr: NativePointer, value: string): void;
};
export declare class TypedPointer<T> {
    readonly type: Type<T>;
    readonly address: StaticPointer;
    constructor(type: Type<T>, address: StaticPointer);
    get(): T;
    set(value: T): void;
    static readonly size: 8;
    static get<T>(type: Type<T>, ptr: NativePointer): TypedPointer<T>;
    static set<T>(ptr: NativePointer, value: TypedPointer<T>): void;
}
export declare type NetworkIdentifierPtr = TypedPointer<NetworkIdentifier>;
export { Primitive, Type, Uint8, Uint16, Uint32, Int8, Int16, Int32, Float32, Float64, };
declare type MakeProperty<T> = {
    [K in keyof T]: [number, K, Type<T[K]>];
};
export interface StructureType<T extends Structure> extends Type<T> {
    new (): T;
    structure: MakeProperty<T>[keyof T][];
}
export declare class Structure {
    static get<T>(this: StructureType<T>, ptr: NativePointer): T;
    static set<T>(this: StructureType<T>, ptr: NativePointer, instance: T): void;
}
export declare class CxxVectorType<T> implements Type<CxxVector<T>> {
    readonly type: Type<T>;
    readonly name: string;
    constructor(type: Type<T>);
    get(ptr: NativePointer): CxxVector<T>;
    set(ptr: NativePointer, value: CxxVector<T>): void;
}
export declare class CxxVector<T> {
    readonly address: StaticPointer;
    readonly type: Type<T>;
    private readonly ctor_move;
    private readonly ctor;
    private readonly dtor;
    constructor(address: StaticPointer, type: Type<T>);
    private _resize;
    set(idx: number, component: T): void;
    get(idx: number): T | null;
    pop(): T | null;
    push(component: T): void;
    size(): number;
}
