"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchAll = exports.close = exports.send = exports.after = exports.before = exports.raw = exports.readLoginPacket = void 0;
const native_1 = require("./native");
const PacketId = require("./packetId");
const krevent_1 = require("krevent");
const common_1 = require("./common");
var readLoginPacket = native_1.nethook.readLoginPacket;
exports.readLoginPacket = readLoginPacket;
class NetEventRaw extends krevent_1.EventEx {
    constructor(id) {
        super();
        this.id = id;
        this.caller = (ptr, size, networkIdentifier, packetId) => {
            const ptrlow = ptr.getAddressLow();
            const ptrhigh = ptr.getAddressHigh();
            for (const listener of this.allListeners()) {
                try {
                    ptr.setAddress(ptrlow, ptrhigh);
                    const ret = listener(ptr, size, networkIdentifier, packetId);
                    if (ret === common_1.CANCEL)
                        return false;
                }
                catch (err) {
                    console.error(err);
                }
            }
        };
    }
    onStarted() {
        native_1.nethook.setOnPacketRawListener(this.id, this.caller);
    }
    onCleared() {
        native_1.nethook.setOnPacketRawListener(this.id, null);
    }
}
class NetEventBefore extends krevent_1.EventEx {
    constructor(id) {
        super();
        this.id = id;
        this.caller = (ptr, networkIdentifier, packetId) => {
            const ptrlow = ptr.getAddressLow();
            const ptrhigh = ptr.getAddressHigh();
            for (const listener of this.allListeners()) {
                try {
                    ptr.setAddress(ptrlow, ptrhigh);
                    const ret = listener(ptr, networkIdentifier, packetId);
                    if (ret === common_1.CANCEL)
                        return false;
                }
                catch (err) {
                    console.error(err);
                }
            }
        };
    }
    onStarted() {
        native_1.nethook.setOnPacketBeforeListener(this.id, this.caller);
    }
    onCleared() {
        native_1.nethook.setOnPacketBeforeListener(this.id, null);
    }
}
class NetEventAfter extends krevent_1.EventEx {
    constructor(id) {
        super();
        this.id = id;
        this.caller = (ptr, networkIdentifier, packetId) => {
            const ptrlow = ptr.getAddressLow();
            const ptrhigh = ptr.getAddressHigh();
            for (const listener of this.allListeners()) {
                ptr.setAddress(ptrlow, ptrhigh);
                listener(ptr, networkIdentifier, packetId);
            }
        };
    }
    onStarted() {
        native_1.nethook.setOnPacketAfterListener(this.id, this.caller);
    }
    onCleared() {
        native_1.nethook.setOnPacketAfterListener(this.id, null);
    }
}
class NetEventSend extends krevent_1.EventEx {
    constructor(id) {
        super();
        this.id = id;
        this.caller = (ptr, networkIdentifier, packetId) => {
            const ptrlow = ptr.getAddressLow();
            const ptrhigh = ptr.getAddressHigh();
            for (const listener of this.allListeners()) {
                ptr.setAddress(ptrlow, ptrhigh);
                listener(ptr, networkIdentifier, packetId);
            }
        };
    }
    onStarted() {
        native_1.nethook.setOnPacketSendListener(this.id, this.caller);
    }
    onCleared() {
        native_1.nethook.setOnPacketSendListener(this.id, null);
    }
}
class CloseEvent extends krevent_1.EventEx {
    constructor() {
        super(...arguments);
        this.caller = (networkIdentifier) => {
            this.fire(networkIdentifier);
        };
    }
    onStarted() {
        native_1.nethook.setOnConnectionClosedListener(this.caller);
    }
    onCleared() {
        native_1.nethook.setOnConnectionClosedListener(null);
    }
}
const rawListeners = [];
const beforeListeners = [];
const afterListeners = [];
const sendListeners = [];
/**
* It will bring raw packet buffers before parsing
* It will cancel the packet if you return false
*/
function raw(id) {
    const event = rawListeners[id];
    if (event)
        return event;
    return rawListeners[id] = new NetEventRaw(id);
}
exports.raw = raw;
/**
* Maybe you cannot find any documents about the parsed packet structure
* You need to discover it self!
*/
function before(id) {
    const event = beforeListeners[id];
    if (event)
        return event;
    return beforeListeners[id] = new NetEventBefore(id);
}
exports.before = before;
/**
* Maybe you cannot find any documents about the parsed packet structure
* You need to discover it self!
*/
function after(id) {
    const event = afterListeners[id];
    if (event)
        return event;
    return afterListeners[id] = new NetEventAfter(id);
}
exports.after = after;
/**
* Maybe you cannot find any documents about the parsed packet structure
* You need to discover it self!
*/
function send(id) {
    const event = sendListeners[id];
    if (event)
        return event;
    return sendListeners[id] = new NetEventSend(id);
}
exports.send = send;
exports.close = new CloseEvent;
/**
 * Write all packets to console
 * @param exceptions (defaults: [
 *                  ClientCacheBlobStatus,
 *                  NetworkStackLatencyPacket,
 *                  LevelChunk,
 *                  ClientCacheMissResponse,
 *                  MoveEntityDelta,
 *                  SetEntityMotion,
 *                  SetEntityData])
 */
function watchAll(exceptions = [
    PacketId.ClientCacheBlobStatus,
    PacketId.NetworkStackLatencyPacket,
    PacketId.LevelChunk,
    PacketId.ClientCacheMissResponse,
    PacketId.MoveEntityDelta,
    PacketId.SetEntityMotion,
    PacketId.SetEntityData,
]) {
    const ex = new Set(exceptions);
    for (let i = 1; i <= 0x88; i++) {
        if (ex.has(i))
            continue;
        before(i).on((ptr, ni, id) => {
            ptr.move(0x28); // skip basic packet datas
            console.log(`R ${PacketId[id]}(${id}) ${ptr.readHex(16)}`);
        });
    }
    for (let i = 1; i <= 0x88; i++) {
        if (ex.has(i))
            continue;
        send(i).on((ptr, ni, id) => {
            ptr.move(0x28); // skip basic packet datas
            console.log(`S ${PacketId[id]}(${id}) ${ptr.readHex(16)}`);
        });
    }
}
exports.watchAll = watchAll;
//# sourceMappingURL=netevent.js.map